@@ -40,6 +40,7 @@ module Data.BinaryList (
   , replicateA
   , replicateAR
   , generate
+  , generateM
     -- * Queries
   , lengthIndex
   , length
@@ -59,13 +60,21 @@ module Data.BinaryList (
   , zip , unzip
   , zipWith
     -- * Lists
+    -- ** From list
   , fromList
   , fromListWithDefault
+    -- ** To list
+  , toListFilter
+  , toListSegment
+    -- * Others
+  , traverseSegment
     -- * Example: Radix-2 FFT
     -- $fft
   ) where
 
-import Prelude hiding ( length,lookup,replicate,head,last,zip,unzip,zipWith,reverse,foldr1,take,map )
+import Prelude hiding ( length,lookup,replicate,head,last
+                      , zip,unzip,zipWith,reverse,foldr1
+                      , take,map,foldr )
 import qualified Prelude
 import Foreign.Storable (sizeOf)
 import Data.List (find)
@@ -76,8 +85,10 @@ import Control.Arrow ((***))
 import Data.Monoid (mappend)
 import Data.Foldable (Foldable (..),toList)
 import Data.Traversable (Traversable (..))
-import Control.Monad.Trans.State (StateT (..),evalState,get,modify)
+import Control.Monad.Trans.State (StateT (..),evalStateT,evalState,get,modify)
+import Control.Monad.Trans.Class (lift)
 import Data.Functor.Identity (Identity (..))
+import Control.Applicative.PhantomState
 
 -- | /O(1)/. Build a list with a single element.
 singleton :: a -> BinList a
@@ -176,6 +187,11 @@ replicate n = runIdentity . replicateA n . Identity
 generate :: Int -> (Int -> a) -> BinList a
 generate l f = evalState (replicateA l $ fmap f get <* modify (+1)) 0
 
+-- | Like 'generate', but the generator function returns a value in a 'Monad'.
+--   Therefore, the result is as well contained in a 'Monad'.
+generateM :: (Applicative m, Monad m) => Int -> (Int -> m a) -> m (BinList a)
+generateM l f = evalStateT (replicateA l $ (get >>= lift . f) <* modify (+1)) 0
+
 -- | /O(log n)/. Get the first element of a binary list.
 head :: BinList a -> a
 head (ListNode _ l _) = head l
@@ -384,7 +400,87 @@ fromListWithDefault e xs =
                       (h:t) -> (h,t)
                       [] -> (e,[])
                ) xs
-        _ -> error "fromListWithDefault: input list is too big."
+        _ -> error "[binary-list] fromListWithDefault: input list is too big."
+
+{-# INLINE toListFilter #-}
+
+-- | /O(n)/. Create a list from the elements of a binary list matching a given
+--   condition.
+toListFilter :: (a -> Bool) -> BinList a -> [a]
+toListFilter c = foldr (\x -> if c x then (x:) else id) []
+
+-- | /O(n)/. Create a list extracting a sublist of elements from a binary list.
+toListSegment :: Int -> Int -> BinList a -> [a]
+{-# INLINE toListSegment #-}
+toListSegment s e xs = runPhantomState (traverseSegment (changeState . (:)) s e xs) []
+
+{-# INLINE traverseSegment #-}
+
+-- | Apply an applicative action to every element in a segment of a binary list, from left to right.
+traverseSegment :: Applicative f => (a -> f ()) -> Int -> Int -> BinList a -> f ()
+traverseSegment f s e xs
+  | s > e = pure ()
+  | e < 0 = pure ()
+  | s >= length xs = pure ()
+  | otherwise = traverseSegmentFromTo f (max 0 s) e xs
+
+{-# INLINE traverseSegmentFromTo #-}
+
+traverseSegmentFromTo :: Applicative f => (a -> f ()) -> Int -> Int -> BinList a -> f ()
+traverseSegmentFromTo f = go
+  where
+    go s e (ListNode n l r) =
+      let k = 2^(n-1)
+      in  if s >= k
+             -- Sublist is contained in right portion
+             then go (s - k) (e - k) r
+             else if e < k
+                     -- Sublist is contained in left portion
+                     then go s e l
+                     -- Sublist is divided in both portions
+                     else traverseSegmentFrom f s l *> traverseSegmentTo f (e - k) r
+    go _ _ (ListEnd x) = f x
+
+{-# INLINE traverseSegmentFrom #-}
+
+traverseSegmentFrom :: Applicative f => (a -> f ()) -> Int -> BinList a -> f ()
+traverseSegmentFrom f = go
+  where
+    go s (ListNode n l r) =
+      let k = 2^(n-1)
+      in  if s >= k
+             -- Sublist is contained in right portion
+             then go (s - k) r
+             -- Sublist is divided in both portions, but right
+             -- portion is taken entirely
+             else go s l *> traverseFull f r
+    go _ (ListEnd x) = f x
+
+{-# INLINE traverseSegmentTo #-}
+
+traverseSegmentTo :: Applicative f => (a -> f ()) -> Int -> BinList a -> f ()
+traverseSegmentTo f = go
+  where
+    go e (ListNode n l r) =
+      let k = 2^(n-1)
+      in  if e < k
+             -- Sublist is contained in left portion
+             then go e l
+             -- Sublist is divided in both portions, but left
+             -- portion is taken entirely
+             else traverseFull f l *> go (e - k) r
+    go _ (ListEnd x) = f x
+
+{-# INLINE traverseFull #-}
+
+traverseFull :: Applicative f => (a -> f ()) -> BinList a -> f ()
+traverseFull f = go
+  where
+    go (ListEnd x) = f x
+    go (ListNode _ l r) = go l *> go r
+
+------------------------------------------------
+------------------------------------------------
 
 -----------------------------
 -- Show and Functor instances
@@ -458,6 +554,4 @@ Fourier Transform of complex vectors with the Radix-2 Fast Fourier Transform alg
 >             BL.append (BL.zipWith (+) evensfft oddsfft)
 >                       (BL.zipWith (-) evensfft oddsfft)
 
-
-
 -}
