@@ -52,7 +52,7 @@ decode input = encodedFromByteString input >>= fromDecoded . decData . decodeBin
 --   If you foresee reading only a part of the list, either
 --   at its beginning or end, an appropiate choice of direction
 --   will allow you to avoid decoding the full list.
-data Direction = FromLeft | FromRight deriving Eq
+data Direction = FromLeft | FromRight deriving (Eq,Show)
 
 -- | A binary list encoded, ready to be written in a file or be
 --   sent over a network. It can be directly translated to a
@@ -103,6 +103,7 @@ data Decoded a = -- | Partial binary list, and rest of decoded input.
                | FinalResult (BinList a) ByteString
                  -- | A decoding error, with an error message and the remaining input.
                | DecodingError String ByteString
+                 deriving Show
 
 -- | Get the final result of a decoding process, unless it returned an error, in which
 --   case this error is returned as a 'String'.
